<?php

// XString.class.php - functions to assist when using strings
//
// $Id: XString.class.php 1424 2008-12-29 16:09:36Z tim $
//
// (c)2000-2008 Tim Jackson (tim@timj.co.uk)
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/**
 * This class contains useful functions when dealing with strings and is
 * primarily intended to be used uninstantiated.
 */
class XString {

	/**
	 * Strip out invalid high-bit ASCII characters used by Microsoft software and replace them with standard ASCII equivalents.
	 *
	 * @param string $string  Input string to clean
	 * @return string  The cleaned string
	 */
	function FilterMS_ASCII ($string) {
		// TODO: use strtr()?
		$trans_table = array (	130 => ',',		// low left rising single quote (looks like a comma)
										132 => ',,',	// low left rising double quote
										133 => '...',	// ellipsis
										145 => "'",		// left single quote				
										146 => "'",		// right single quote
										147 => '"',		// left double quote
										148 => '"',		// right double quote
										149 => '·',		// round filled bullet &middot
										150 => '-',		// en dash (like a hyphen)
										151 => '--',	// em dash (like a double hyphen)
										152 => '~');	// small spacing tilde
		foreach ($trans_table as $char => $replacement) {
			$string = str_replace (chr($char), $replacement, $string);
		}	
		return $string;
	}
	
	// Insert paragraph breaks - should catch both double <cr><lf> and <lf>
	function AddPBreaks ($string) {
		$string = str_replace("\r\n\r\n","<p>\n",$string);
		$string = str_replace("\n\n","<p>\n",$string);
		return $string;
	}
	
	//Insert line breaks - should catch both <cr><lf> and <lf>
	function AddLBreaks ($string) {
/*		$string = str_replace("\r\n","<br>\n",$string);
		$string = str_replace("\n\n","<br>\n",$string);
		return $string;*/
		return nl2br($string);
	}
	
	// Outputs to HTML
	// string XString::Output(string string)
	function OutputHTML ($string) {
		global $PHPOF_OUTPUT_CHARSET;
		return str_replace(chr(128),'&euro;',stripslashes(XString::AddLBreaks(htmlentities($string,ENT_QUOTES,$PHPOF_OUTPUT_CHARSET))));
	}
	
	function OutputXML ($string) {
		global $PHPOF_OUTPUT_CHARSET;
		return stripslashes(XString::AddLBreaks(htmlspecialchars($string,ENT_QUOTES,$PHPOF_OUTPUT_CHARSET)));
	}
	
	// Backwards compatibility
	function Output ($string) {
		return XString::OutputHTML($string);
	} 
	
	// Outputs plain texts
	function OutputText ($string) {
		$trans = get_html_translation_table (HTML_ENTITIES);
		$trans = array_flip($trans);
		$string = strtr($string, $trans);
		$string = stripslashes($string);
		$string = strip_tags($string);
		return $string;
	}
	
	function HTML2Text ($String)
	{
		global $PHPOF_OUTPUT_CHARSET;
		
		// replace linebreaks and tabs in the initial HTML with a single space - remember line
		// breaks in HTML are equivalent merely to a space
		$String = str_replace("\n",' ',$String);
		$String = str_replace("\r",' ',$String);
		$String = str_replace("\t",' ',$String);
		
		// replace multiple spaces in source HTML with a single space; that's how HTML works
		$String = preg_replace("/ +/",' ',$String);
		
		// Get rid of stuff within style/script tags - we don't want that
		$String = preg_replace("/<style.*\/style>/Us",'',$String);
		$String = preg_replace("/<script.*\/script>/Us",'',$String);
		
		// Paragraph breaks get two newlines
		$String = preg_replace("/<p[^>]*>/i","\n\n",$String);
		
		// TODO: clever table handling
		// For now, split table cells witha single space
		$String = preg_replace("/<\/td>/i",' ',$String);
		
		// Replace <br>s with a single newline
		$String = preg_replace("/(<br(\s*\/)?\s*>)/i","\n",$String);
		
		// Emulate bullet points on lists
		// TODO: do ordered lists properly
		$String = preg_replace("/<li[^>]*>/i","\n * ",$String);
		$String = preg_replace("/<(ul|ol)[^>]*>/i","\n",$String);
		
		// Use ALT attributes in place of images
		//$String = preg_replace("/<img[^>]* alt=(['\"])(.*)(\\1)[^>]*>/i",' \2 ',$String);
		
		// Now strip out all tags
		$String = strip_tags($String);
		
		// Trim leading space on lines - breaks lists
		$String = preg_replace("/\n +/","\n",$String);
		// Unbreak lists again
		$String = str_replace("\n*","\n *",$String);
		
		// PHP <5 does not support multi-byte character sets in html_entity_decode()
		// See http://bugs.php.net/bug.php?id=25670
		if (phpversion() > 5 || strtolower($PHPOF_OUTPUT_CHARSET) == 'iso-8859-1') {
			$String = html_entity_decode($String, ENT_QUOTES, $PHPOF_OUTPUT_CHARSET);
		} else {
			$EntityTable = array_flip(get_html_translation_table(HTML_ENTITIES));
			$EntityTable['&bull;'] = '-';
			$String = strtr($String, $EntityTable);
			if (strtolower($PHPOF_OUTPUT_CHARSET) == 'utf-8') {
				$String = preg_replace('/&#(\\d+);/e', 'XString::code2utf($1)', utf8_encode($String));
			}
		}
		
		// Do this twice to make sure we got rid of everything
		$String = preg_replace("/\n{3,}/","\n\n",$String);
		$String = preg_replace("/\n{3,}/","\n\n",$String);
		
		// Final trim
		$String = trim($String);
		
		return $String;
	}
	
	// Thanks http://bugs.php.net/bug.php?id=25670
	function code2utf($num)
	{
		if ($num < 128) return chr($num);
		if ($num < 2048) return chr(($num >> 6) + 192) . chr(($num & 63) +128);
		if ($num < 65536) return chr(($num >> 12) + 224) . chr((($num >> 6) & 63) + 128) . chr(($num & 63) + 128);
		if ($num < 2097152) return chr(($num >> 18) + 240) . chr((($num >> 12) & 63) + 128) . chr((($num >> 6) & 63) + 128) . chr(($num & 63) + 128);
		return '';
	}

	// Useful when dumping form-submitted data back out to HTML without going
	// via a database
	function Form ($string) {
		global $PHPOF_OUTPUT_CHARSET;
		if (!isset($PHPOF_OUTPUT_CHARSET) || $PHPOF_OUTPUT_CHARSET == null) {
			trigger_error('Please define PHPOF_OUTPUT_CHARSET',E_USER_ERROR);
		}
		                                        
		return htmlentities(stripslashes($string),ENT_QUOTES,$PHPOF_OUTPUT_CHARSET);
	}
	
	function getFileExtension ($String) {
		preg_match('/.*\.(.*)$/',$String,$Matches);
		if (isset($Matches[1])) {
			return trim($Matches[1]);
		}
		return false;
	}
	
	// Truncate the given text after $Paras paragraphs
	function truncateBySentence ($Text, $Sentences)
	{
		if (!empty($Text)) {
			// Deal with ellipses
			$TempText = str_replace('...','%{point}%{point}%{point}',$Text);
			
			// Deal with "circa" text, e.g. " foobar c.1710 flibble"
			$TempText = preg_replace("/\b(c)\.(\s?)(\d+)\b/i",' \1%{point}\2\3 ',$TempText);
			
			// Deal with floating point numbers
			$TempText = preg_replace("/\b(\d+)\.(\d+)\b/", ' \1%{point}\2 ',$TempText);
			
			// Deal with hostnames
			$TempText = preg_replace_callback('/([a-z0-9-]{1,40}\.){1,4}[a-z0-9]{2,40}/', array('XString','tscallback'),$TempText);
			
			$Pos = -1;
			for ($x = 1; $x<=$Sentences; $x++) {
				$NewPos= strpos($TempText,'.',$Pos+1);
				$NewPos2 = strpos($TempText,'!',$Pos+1);
				//print "FOUND $x: pos1=$NewPos,pos2=$NewPos2   ";
				if ($NewPos === false && $NewPos2 === false) {
					//print "break at $x";
					$Pos = false;
					break;
				} else {
					// If no exclamation mark found, or full stop occurred before it
					if ($NewPos2 === false || $NewPos < $NewPos2) {
						$Pos = $NewPos; // Use full stop as sentence terminator
					} else {
						$Pos = $NewPos2; // Use exclamation mark as sentence terminator
					}
				}
				//print "finalpos=$Pos<br />   ";
			}
			
			if ($Pos !== false) {
				return str_replace('%{point}','.',substr($TempText,0,$Pos+1));
			}
			return $Text;
		}
	}
	
	function tscallback($Matches)
	{
		return str_replace('.','%{point}',$Matches[0]);
	}
	
	function truncateByParagraph ($Text, $Paras, $Offset=0) {
		if (!empty($Text)) {
			$StartPos = 0;
			$TempText = str_replace("\r",'',$Text);
			$Pos = -1;
			for ($x = 1; $x<=($Paras+$Offset); $x++) {
				$Pos= strpos($TempText,"\n\n",$Pos+1);
				if ($x == $Offset) {
					$StartPos = $Pos;
				}
				if ($Pos === false) {
					// If offset is more than the total number of paras, return nothing
					if ($Offset >= $x) {
						$StartPos = strlen($TempText);
					}
					break;
				}
			}
			
			if ($Pos !== false) {
				return substr($TempText,$StartPos,($Pos+1-$StartPos));
			}
			return substr($Text,$StartPos);
		}
	}
	
	// TODO: try this with a preg_replace, see if it's faster
	function recursiveReplace($SearchFor, $ReplaceWith, $String)
	{
		$NewString = '';
		while ($NewString != $String) {
			if ($NewString != '') $String = $NewString;
			$NewString = str_replace($SearchFor, $ReplaceWith, $String);
		}
		return $String;
	}
	
	function stringToJsLiteral($String)
	{
		$String = str_replace("\n", '\n', $String);
		$String = str_replace("\r", '', $String);
		return $String;
	}
	
	function trimAll($Array)
	{
		if (is_array($Array) && count($Array) > 0) {
			foreach ($Array as $Key => $Value) {
				$Array[$Key] = trim($Array[$Key]);
			}
		}
		return $Array;
	}
	
	function toURL($String, $Separator='-')
	{
		$String = strtolower(str_replace(' ',$Separator,$String));
		$String = strtolower(str_replace('/',$Separator,$String));
		$String = preg_replace("/[^[:alnum:]$Separator]/",'',$String);
		$String = preg_replace("/([\s$Separator]){2,}/",'\1',$String);
		return $String;
	}
	
	function linearise($String)
	{
		$OldString = $String;
		$String = str_replace("\n", ' ', $String);
		$String = str_replace("\t", ' ', $String);
		$String = str_replace("\r", ' ', $String);
		$String = preg_replace('/[\s_=-]{2,}/m',' ',$String);
		$String = trim($String);
		if ($String != $OldString) {
			$String = XString::linearise($String);
		}
		return $String;
	}
}
